/* geosat10hz.h

W H F Smith, 7 Aug 2004

As with sdrwdr.h, things are indicated as "unsigned" as a clue to valid
physical range or meaning, but I think in most cases the most significant
bit is not used, so it is safe to read these into an ordinary integer for
fortran programmers.  
Exceptions are that the unsigned char waveform must be interpreted as such,
and the FLAG_NAN is 2^31, which will be a negative value in a signed int.

Below the values given twice[2] are for a two-pass retracker, which first
allows all parameters to be free and later retracks range only using smoothed
versions of the other parameters.

*/

/*#include "sdrwdr.h"*/

struct GEOSAT10HZ {	/* Merges stuff in ICD Tables 2 and 5 at 10 Hz, plus retracking info. */

	unsigned int	kframe;		/* Frame counter.
					This is a unique integer that increases by 1
					at each 0.098... second waveform value.
					kframe = 32*(Major frame count) + (Minor frame count).
					Combines items 1 and 2 of Table 2 or 5.  */
	
	unsigned int	tai85s;		/* atomic seconds since 1985  */
	unsigned int	nanos;		/* nanoseconds since tai85s  (See NOTE ONE of sdrwdr.h) */
	unsigned int	alt;		/* orbit altitude, mm  (SEE NOTE ONE of sdrwdr.h) */
	unsigned int	range;		/* range, per on-board tracker, mm (item 5 thru 14) */
	unsigned int	modebits;	/* "mode" word; (item 4, only least-significant 30 bits used) */
	unsigned int	flagbits;	/* "flagword";  (item 5, only least-significant 30 bits used) */
	unsigned int	trackbits;	/* Useful flags to be defined as we go along; initially zero  
                                          1  - waveform unavailable
                                          2  - failed fibtrack5 or 4, retracking
                                          4  - dry land
                                          8  - pnoise < 30 or pnoise > 180
                                          16 - pagc < 1300 or pagc > 2100
                                          32 - pswh < 100 or pswh > 12000
                                          64 - chisq(1) > 3000
                                         128 - failed fibtrack1
                                         256 - chisq(2) > 1500  uses weight = (5 + power)/10 
                                         512 - decay(2) > 5600  */

	int		lon;		/* degrees * 10^6 or * 10^7 if -180 to 180  */
	int		lat;		/* degrees same units as lon */
	int		mss;		/* model mean sea surface (CLS00.1 ?), mm  */
	int		ispare;		/* spare not yet defined rounds this part to 48 bytes  */
	
	unsigned short int pswh[2];	/* pseudo-swh, waveform spread estimated by retracking, mm  */
	unsigned short int pagc[2];	/* pseudo-agc, waveform power  estimated by retracking x 10 */
	unsigned short int rchisq[2];	/* root sum of squares of misfit error from tracker  x 100 */
	unsigned short int pnoise[2];	/* noise level preceding leading edge, est by retracking x 10 */
	short int	drange[2];	/* range correction from retracking, mm  */
	short int	decay[2];	/* plateau decay rate est by retrack, prop to xi  */
	
	unsigned short int ppp;		/* pulse peakiness parameter, x1000  */
	unsigned short int swh;		/* swh from on-board tracker, 0 to 2000 cm (item 18 thru 27) */
	unsigned short int agc;		/* agc (on-board), 0 to 6400 units of 0.01 dB (item 30 to 39)  */
	unsigned short int xi;		/* off-nadir angle, 0 to 200 units of 0.01 degree.  (item 45)
					This is derived from vatt and may not be useful. */
	unsigned short int sig0;	/* sigma-naught, 0 to 6400 units of 0.01 dB.  (item 15)  
					This is computed from 1-Hz avg agc using a naive constant 
					altitude.  May not be useful once we understand how to go
					from agc to sig0.  */
	unsigned short int wc;		/* Wind speed, 0 to 320 units of 0.1 m/s.  (item 47)
					This is derived from sig0 and may not be useful to carry.  */
	unsigned short int vatt_t;	/* voltage proportional to attitude, 0 to 3000 millivolts.  (item 48)
					Derived including cal correction but not 60-sec smoother?  */

	short int	dhsx;		/* height correction from (SWH|xi) lookup, +/-9999 mm (item 16) */
	short int	dhfm;		/* height correction from pulse fm error, +/-999 mm (item 17) */
	short int	dswhsx;		/* swh correction from (SWH|xi) lookup, +/-999 cm  (item 29) */
	short int	dagcsx;		/* agc correction from (SWH|xi) lookup, +/-999 units of 0.01 dB (item 41)  */
	short int	dagch;		/* agc correction for height, +/-999 units of 0.01 dB (item 42)  */
	short int	dagct;		/* agc correction for temperature, +/-999 units of 0.01 dB (item 43)  */
	short int	rcvt0;		/* receiver temperature, -300 to +1100 units of 0.1 degrees C (item 49)  */
	short int	dhdt;		/* dh/dt estimate from on-board range rate, -99 to +99 m/s (item 44)  */
	
	short int	hotide;		/* (pure, only) ocean tide height, mm.  From model (FES2002?).  */
	short int	hltide;		/* load tide height, mm.  From model (FES2002?).  */
	short int	hstide;		/* solid earth tide, mm.  From model.  */
	short int	hptide;		/* height correction for pole tide, mm.  From model.  */
	short int	hiono;		/* height correction for iono delay, mm.  From a model.  */
	short int	hwet;		/* height correction for wet tropo delay, mm.  From a model.  */
	short int	hdry;		/* height correction for dry tropo delay, mm.  From a model.  */
	short int	hinvb;		/* height corr for inverse barometer, mm.  From model  */
						
	short int	hssb;		/* height corr for sea state bias, from a model (to be fit ?).  */
	short int	pdopp2;		/* Not given.  Space to store a home-grown doppler correction
					to range.  The ICD claims that a 1-Hz avg of on-board-tracked
					range-rate is sufficient for the doppler correction, but Chelton
					et al (AP, 2001) indicate more smoothing needed due to noise.
					Required is target range rate, not orbit height rate, because
					geoid slope in trenches is significant at the mm level.  */
	short int	gindex;		/* Not given in the SDR or WDR but perhaps guessable from
					corrections depending on SWH and xi in ICD Table A-1.  [1 thru 5].
					This may be useful for sorting out on-board tracker behavior,
					and is a parameter needed to follow the recipes for waveform
					calculations and corrections to on-board estimates in the ICD.  */
	short int	trackmode;	/* 1 thru 4 for a valid track mode, else zero  */
	short int	seq10;		/* 0 thru 9 indicating sequence number in 1-Hz frame  */
	
	unsigned char	wave[64];	/* waveform data.  First 60 locations are the 60-gate waveform.
					Next three are the halfway-in-between fine-tracking gates.
					(These 63 are stored consecutively in the same way in the WDR,
					as an array(63,10) (fortran order) beginning as Item 5 of Table 5.)
					Last location is the dividing factor (1,2 or 4) used to scale
					waveform down to fit in unsigned char (done on board).
					(These are stored as items 635 to 644 of Table 5)  */
};
