/* takes the Cryosat ocean waveform and then retracks the leading edge */

/*-- standard C headers --*/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>

/*--
   routine to take the Cryosat ocean waveform and re-track the
   leading edge
   
   input
     nw       -   odd number of waveforms in the least squares fit
     sigw     -   sigma of Gaussian weight function used in fit
     rw       -   512 samples of the waveform
     iwf      -   (0) - use waveform  (1) - don't use waveform
     t0       -   initial estimate of arrival time (1-512)
     sig0     -   initial estimate of pulse width
     npar     -   (2) - solve for amp and t1 (3) - also solve for sig1

   output
     rmw      -   model waveform
     t1       -   updated arrival time (1-512)
     sig1     -   updated pulse width
     amp      -   model amplitude
     std      -   standard deviation of misfit (< 0 -  something went wrong)
     prob     -   F-test probability that variance reduction is due
                  to random fluctuations (< 0.2 keep iterating)

--*/

int sinan(nw,sigw,rw,iwf,t0,sig0,npar,alpha,rmw,t1,sig1,amp,std,prob,chi2)
int 	*nw; /*---input parameters passed as pointers---*/
double	*sigw;
double	*rw;
int	*iwf;
double	*t0;
double	*sig0;
int	*npar;
double *alpha;
double	*rmw; /*---output parameters to pass as pointers---*/
double	*t1;
double	*sig1;
double	*amp;
double	*std;
double	*prob;
double	*chi2;
{

/*-- dereference the pointers to variables that are internal to the function --*/

	/*-- input parameters --*/
	int		npari=*npar;
	int		mx=512*(*nw);

/*-- internal variables --*/
	int		dpt, gate, info, nc, ndata, np, nt, parnc, parnr, wavno, nw2;
	double	chi1, f, suma, sumb, summ, sumw, talph, sigw2, time, wfac, z;
	double	ex[512], dmds[mx], dmdt[mx], rm[mx];
	double	a[mx][3];
	double	b[mx], w[mx];
	double	ata[3][3];
	double	atb[3];
	char	uplo;
	
/*-- set some constants --*/
	double	damp=0.9;
	double	sq2=sqrt(2.0);
	
/*-- set some parameters specific to Cryosat --*/
	int		mingate, maxgate;
	
/*-- new to SAR retracking --*/
	double	eta, dgd0;
	double	gd0, mfac, om, od, pcf, pcd;
	double	dva[2], dvp[2], ggda[2], dsa, dta;


	nw2 = (*nw-1)/2;

/*-- make sure there is enough memory --*/
	
	if ((*nw)*512 > mx) {
		fprintf(stderr,"Increase mx in sinan");
	}
	

	
/*-- don't let the swh get less than 0.54 m --*/
	if(*sig0 < 0.3){
		*sig0=0.3;
	}

        /* use the new derivation */
	
	om = -.5;
	od = .5;
	
	mingate=28;
	maxgate=484;
	
	ggda[0] = 0.0;
	ggda[1] = 0.0;
	dsa = 0.0;
	dta = 0.0;
		
/*-- construct the initial model and partial derivatives --*/
	for(wavno = 0; wavno<*nw; wavno++){
		for(gate=0; gate<512; gate++){
			dpt = 512*(wavno)+gate;
			time = ((double)gate)+1.0;
			talph = fmax(0.0,(time-t0[wavno])*(*alpha));
			eta = -(time-t0[wavno])/(*sig0);
		

			ex[gate] = exp(-talph);
		
			mfac=1.0/sqrt(2.0);
		
			pbdv_(&om, &eta, dva, dvp, &pcf, &pcd);
			gd0 = pcf;
		
			rm[dpt] = ex[gate]*(mfac*gd0/sqrt(*sig0));	
		
			pbdv_(&od, &eta, dva, dvp, &pcf, &pcd);
			dgd0 = pcf;
		
			dmds[dpt] = (mfac/(sqrt(*sig0)*(*sig0)))*(-(0.5*gd0)+(eta*dgd0));
			dmdt[dpt] = -(mfac/(sqrt(*sig0)*(*sig0)))*dgd0;		
		}
	}
	
/*-- estimate the amplitude --*/

	sumw = 0.0;
	summ = 0.0;
	nc=((*nw+1)/2)-1;
	sigw2 = (*sigw)*(*sigw);
	
/*-- Initialize A, b, and w --*/
	
/*-- this weight function depends on the distance to the center of the 
	window. also make this large if this waveform is flagged as bad. --*/
	
	for(wavno = 0; wavno<*nw; wavno++){
		wfac=exp(((double)(wavno-nc)*(double)(wavno-nc))/(2.0*sigw2));
		if(iwf[wavno] == 1) {
			wfac=100000.0;	
		}
		// set value of mingate and maxgate depending on t0
		for(gate = mingate; gate<maxgate+1; gate++){
			dpt = 512*(wavno)+gate;

/*-- Use uniform weight for SAR and SIN data --*/
			w[dpt] = wfac*1250.0; 
			sumw = sumw + ((rw[dpt]*rm[dpt]))/(w[dpt]*w[dpt]);	
			summ = summ + ((rm[dpt]*rm[dpt]))/(w[dpt]*w[dpt]);	
		}
		
	}
	
	*amp = sumw/summ;

/*-- Modify and subtract the starting model and construct the A-matrix 
	 (which is the diagonal weight matrix W multiplied to the Jacobian matrix J
	 (which in turn is contains partial derivatives of the model function 
	 with respect to the model parameters)) --*/
	 
	 chi1  = 0.0;
	 	 
	 for(wavno = 0; wavno<*nw; wavno++){
	 	for(gate = mingate; gate<maxgate+1; gate++){
	 		dpt = 512*(wavno)+gate;
	 		b[dpt] = (rw[dpt] - (*amp)*rm[dpt])/w[dpt];
	 		chi1 = chi1 + (b[dpt]*b[dpt]);
	 		a[dpt][0] = rm[dpt]/w[dpt];
	 		a[dpt][1] = (*amp)*dmdt[dpt]/w[dpt];
	 		a[dpt][2] = (*amp)*dmds[dpt]/w[dpt];
	 	}
	 }

/*-- Compute (A^T)A and (A^T)b --*/

	/*-- (A^T)b --*/
	for(parnr = 0; parnr<*npar; parnr++){
	
		sumb = 0.0;
		
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
				dpt = 512*(wavno)+gate;
				sumb = sumb + (a[dpt][parnr]*b[dpt]);
			}
		}
		
		atb[parnr] = sumb;

/*-- (A^T)A --*/	
	for(parnc = parnr; parnc<*npar; parnc++){

		suma = 0.0;
		
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
				dpt = 512*(wavno)+gate;
				suma = suma + (a[dpt][parnr]*a[dpt][parnc]);
			}
		}
		ata[parnc][parnr] = suma; /*-- This is the transpose of the actual result --*/
	}
	}
	

/*-- Solve the problem --*/
	
	uplo='U';
	np=3;
	nt=1;
	
	dposv_(&uplo,npar,&nt,ata,&np,atb,&np,&info);

/*-- add comments here about the arguments --*/
	
/*-- construct the solution --*/
	if(info == 0){
		*amp = *amp+damp*atb[0];
		
		for(wavno = 0; wavno<*nw; wavno++){
			t1[wavno] = t0[wavno]+damp*atb[1];
		}
		
		if(npari == 2){
			*sig1 = *sig0;
		} else {
			*sig1 = *sig0+damp*atb[2];
			if(*sig1 < 0.1){
				*sig1 = 0.1;
			}
		}
	
					
/*-- generate the final model --*/
	for(wavno = 0; wavno<*nw; wavno++){
		for(gate = 0; gate<512; gate++){
			dpt = 512*(wavno)+gate;
			time = ((double)gate)+1.0;
			talph = fmax(0.0,(time-t1[wavno])*(*alpha));
			eta = -(time-t1[wavno])/(*sig1);
			ex[gate] = exp(-talph);
	                mfac= 1.0/sqrt(2.0);
			pbdv_(&om, &eta, dva, dvp, &pcf, &pcd);
			gd0 = pcf;
			rm[gate]=ex[gate]*(mfac*gd0/sqrt(*sig1));	
			rmw[dpt]=rm[gate]*(*amp);
		}	
	}	
	
/*-- evaluate the misfit --*/
		*chi2 = 0.0;
		ndata = 0;
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
	 			dpt = 512*(wavno)+gate;
	 			ndata = ndata+1;
	 			*chi2 = *chi2 + ((rw[dpt]-rmw[dpt])*(rw[dpt]-rmw[dpt])/(w[dpt]*w[dpt]));			
	 		}
	 	}
		*std = sqrt(*chi2/(ndata-npari));
		f = (*chi2)/chi1;
		z = (f-1.046511628)/0.323222961;
		*prob = (1.0+erf(z*sq2))/2.0;
	} else {
		*std = -abs(info);
	}
	
	return EXIT_SUCCESS;
}
