!
      program j20_cdr
!
!          Copyright 2011
!	   David T. Sandwell
!
!  program to read cryosat altimeter data in j20 format.
!  the files are reformatted into binary direct access files.
!  the 20 per second data are low-pass filtered and then
!  compressed into 5/sec data.
!
!  1. imode = 0 use all threshold retracked data having abs(lat) > 60.
!     imode = 1 use 2-parameter retracked data edited for the following reasons:
!      land data
!      flag    flag turned on
!      swh     exceeds 8.0 m or less than 0.5 m
!      amp     exceeds 80000  or less than 40000
!      chisq1  exceeds 0.70
!      |swh1 - swh0| exceeds 900
!
!  2. reading continues until data time points are
!     separated by more than 4 seconds and/or delta
!     theta between successive points switches sign.
!     these conditions define a new pass.
!
!  3. five data points are created for each second
!     by filtering adjacent points from the 20/sec data.
!     slopes between data adjacent points and computed
!     using a first difference formula and stored with
!     the second time point. the very first slope in the array
!     is assigned a value the # of
!     points in the pass.   this value signifies the
!     begining of a pass.
!
!  4. the following corrections are summed and added to the height data.
!      solid earth tide
!      ocean tide
!      doppler correction
!      A 0% SSB is applied to the height.
!
!  5.  The sum of the atmospheric corrections is stored in the last cdr area
!      wet troposphere fnoc
!      dry troposphere
!      inverted barometry
!
!  6. each pass with more than 20 points is then dumped
!     into an ascending or descending binary file.
!
!     external funcs
      parameter (ND=200000,MP=1000,NP=50)
      implicit real*8 (d)

include "jason20hz.inc"

      type(reaa) rea
      common /iemas/irec(7*17000)
      common /info/ lcrt(2),lin(10),nin,lout(10),nout
      common /count/ncount(2)
      common /emas/ dtm(17000),rlt(17000),rln(17000),rsh(17000),rdsh(17000),rfsh(17000),rcor(17000)
      real*8  dm(ND),drt(ND),drn(ND)
      real*4  rh(ND),rswh(ND),rhout(ND),rcr(ND)
!     real*4  rswh0(ND),rchisq(ND),rswhs(ND)
      real*8  dflt(99)
      integer ddrange
!
!  filter - 99 point, Park-McClellan low-pass filter
!
include "filter.inc"
!
!  constants
!
      data td/1.01/
      data rad/.01745329e0/
      data imode/0/
!
!   get the output mode
!
      write(*,900)
 900  format(' enter output mode: 0-threshold; 1-2parameter: ',$)
      read(*,*)imode
!
!  initialize counters
!
      ncount(1)=1
      ncount(2)=1
!
!  open binary input file for reading
!
      call inioc(lu)
!
!  open ascending and descending output files
!
      write(lcrt(2),901)
 901  format(' ascending, ',$)
      call outioc(lu1)
      write(lcrt(2),902)
 902  format(' descending, ',$)
      call outioc(lu2)
!
!  should the file header be written?
!
      write(lcrt(2),904)
 904  format(' write file header (1-yes) (0-no):  ',$)
      read(lcrt(1),*)iwrthead
!
!  the type is ers/envisat
!
      ityp=9
!
!   output files are integers
!
      ifile=1
!
!  time increment is 5 Hz.
!
      idtt=1.d06*td/5.
      if(iwrthead.eq.1) then
!
!  write the satellite information to the output files
!
      iasc=1
      call writhead(lu1,ityp,iasc,ifile,idtt)
      iasc=2
      ncount(2)=1
      call writhead(lu2,ityp,iasc,ifile,idtt)
! 
!  reset both counters to  the second record
!
      ncount(1)=2
      ncount(2)=2
      endif
! 
! initialize other counters
!
      mdata=0
      nflag=0
      nswh=0
      ndswh=0
      namp=0
      nchisq=0
      nlat=0
      nland=0
!
!  offset based on fit to EGM2008
!
      ioff = -13360
!
! read a pass
!
  15  continue
      ip=1
!
      if(mdata.eq.0)then
      ip=0
      endif
!
!***********load the pass************************
!
  20  continue
!
!  read a 400-byte record.
!
      call rddiscb(lu,rea,400,istin)
      if(istin.lt.0) go to 50
!  
!  get the variabiles used for editing
!
      iflag=rea%trackbits
      swh0=rea%pswh(1)/1000.
      swh1=rea%pswh(2)/1000.
      dswh=abs(swh1-swh0)
      amp=float(rea%pamp(2))
      chisq=rea%rchisq(2)/1000.
      dalt=abs(rea%lat*1.e-06)
      dlat=rea%lat*1.e-06
      dlon=rea%lon*1.e-06
      idry=iwetdry(dlat,dlon)
!
!  editing based on mode
!
      if(imode.eq.1) then
!
!  edit of not water
!
      if(idry.eq.2) then
      nland=nland+1
      go to 20
      endif
!
!  edit if any of the flagbits is on
!
      if(iflag.gt.2.or.iflag.eq.1) then
      nflag=nflag+1
      go to 20
      endif
!
!  edit if swh exceeds 8.0 m or is less than 0.5 m
!
      if(swh1.gt. 8.0.or.swh1.lt.0.5) then
      nswh=nswh+1
      go to 20
      endif
!
!   edit if the 3-par swh and smoothed swh differ by more than 0.9 m
!
      if(dswh.gt.0.9) then
      ndswh = ndswh +1
      go to 20
      endif
!
!  edit if amp exceeds 80000 or is less than 40000
!
      if(amp.gt.80000.or.amp.lt.40000) then
      namp=namp+1
      go to 20
      endif
!
!  edit if chisq exceeds 0.80
!
      if(chisq .gt. 0.8) then
      nchisq=nchisq+1
      go to 20
      endif
!
      else
!
!  mode = 0 for threshold data
!
      if(idry.eq.2) then
      nland=nland+1
      go to 20
      endif
!
      if(dalt.lt.60.) then
      nlat=nlat+1
      go to 20
      endif
!
      endif

!
! keep the record
!
      mdata=mdata+1
      ip=ip+1
!
!  decode a record
!
      dm(ip)=rea%sec2000+rea%microsec*1.d-06
      drt(ip)=rea%lat*1.e-06
      drn(ip)=rea%lon*1.e-06
      rswh(ip)=rea%pswh(2)*1.e-3
!
!   apply all of the corrections and tide.  
!
      tide = rea%hltide + rea%hstide + rea%hptide
      if(rea%new_tide .ne. 32767) tide = tide + rea%new_tide
!      delay = rea%hwet + rea%hdry + rea%hiono + rea%hinvb  iono adds noise
      delay = rea%hwet + rea%hdry + rea%hinvb 
      rcr(ip) = delay*1.e-3 
!
!  use threshold if imode=0 else use 2-parameter 
!
      ddrange = rea%drange(3)
      if(imode.eq.1) ddrange = rea%drange(2)
      rh(ip)=(ioff+rea%alt-(rea%range+ddrange+tide))*1.e-3
!
!  apply a 0% SSB based on fit to EGM2008
!
!     rh(ip)=rh(ip)+0.00*rswh(ip)
!
!   subtract the SSH model for evaluation purposes
!
!     rh(ip)=rh(ip)-rcr(ip)-rea%mss*1.e-3
!
!
!  start filling the temp arrays
!
      if(ip.eq.1) go to 20
!
!  check for too many points
!
      if(ip.gt.ND) then
      write(lcrt(2),905)
 905  format(' warning ! # of points exceeded ')
      endif
!
!  check for change in pass orientation
!
      if(ip.eq.2)iasc0=sign(1.D0,drt(2)-drt(1))
      iasc=sign(1.D0,drt(ip)-drt(ip-1))
      if(iasc.ne.iasc0) go to 50
!
!  check for significant time gap
!
      ddtm=dm(ip)-dm(ip-1)
      if(ddtm.gt.4.d0.or.ddtm.le.0.d0) go to 50
      go to 20
!
!*********filter and write the pass**************************
!
   50 continue
!
!  omit short passes < 80 points
!
      ipm1=ip-1
      if(ipm1.le.80) go to 70
!
!  filter the 20/s data
!
      ik=0
      do 60 k=1,ipm1,4
      dsum=0.
      dsumw=0.
      do 55 l=-49,49
      kl=k+l
      kl=max(kl,1)
      kl=min(kl,ipm1)
      dsumw=dsumw+dflt(l+50)
      dsum=dsum+dflt(l+50)*rh(kl)
   55 continue
      if(dsumw.gt.0.8) then
        rhout(k)=dsum/dsumw
      else
        rhout(k)=-999.
      endif
!     write(*,903)rh(k),rhout(k)
!903  format(3f8.4)
!
!  dump the filtered data at 5 Hz
!
      if(rhout(k).lt.-990.) go to 60
      ik=ik+1
      dtm(ik)=dm(k)
      rlt(ik)=drt(k)
      rln(ik)=drn(k)
      rsh(ik)=rhout(k)
      rcor(ik)=rswh(k)
      if(ik.gt.1) then
         ddtm=dtm(ik)-dtm(ik-1)
         rdsh(ik)=(rsh(ik)-rsh(ik-1))/ddtm
      endif
 60   continue
!
!  the output array is now filled.
!  put the real data into an integer array and
!  write out the pass on the appropriate file.
!
      ik1=ik
      call realint(1,1,ik1)
      irec(6)=ik1
      if(iasc0.eq.1) then
        lott=lu1
        indx=1
      else if(iasc0.eq.-1) then
        lott=lu2
        indx=2
      else
        write(*,'(a)') '***** error iasc0'
        stop
      endif
      if(ik1.le.20) go to 70
      call wrdisc(lott,irec,7*ik1)
      ncount(indx)=ncount(indx)+ik1
      write(lcrt(2),907)ik1,iasc0
  907 format('  # of points,  ascending(1) or descending(-1) ; ',2i6)
!
!  stuff the last record read into the first slot.
!
  70  continue
      if(istin.lt.0) go to 999
      dm(1)=dm(ip)
      drt(1)=drt(ip)
      drn(1)=drn(ip)
      rswh(1)=rswh(ip)
      rh(1)=rh(ip)
      rcr(1)=rcr(ip)
      go to 15
!
!  close the files.
!
 999  continue
      call closio(-1)
      write(lcrt(2),912)nflag,nswh,namp,nchisq,ndswh,nlat,nland
 912  format(/,'edited: flag, swh, amp, chisq, dswh, nlat, nland  = ',7i9)
      write(lcrt(2),911)mdata
 911  format(i12, ' data points not edited ')
      nasc=ncount(1)-1
      ndes=ncount(2)-1
      write(lcrt(2),913)nasc,ndes
 913  format(' # ascending file, # descending file ',2i12)
      stop
      end
!
!
      block data luio
!
!  common blocks for logical unit # transfers.
!
      common/info/lcrt(2),lin(10),nin,lout(10),nout
!
!   lcrt(1)  -  lu for terminal read.
!   lcrt(2)  -  lu for terminal write.
!   lin(i)   -  i'th input file lu (10-19 only).
!   nin      -  # of input files opened.
!   lout(i)  -  i'th output file lu (20-29 only).
!   nout     -  # of output files opened.
!
!***** calls no other routines
!
      data lcrt/5,6/
      data lin,nin/10*5,0/
      data lout,nout/10*6,0/
      end

