c
      subroutine erfpick(nw,sigw,iw,iwf,t0,sig0,npar,
     +                    rmw,t1,sig1,amp,std,prob)
c123456789012345678901234567890123456789012345678901234567890123456789012
c
c
c   routine to take the Cryosat ocean waveform and re-track the
c   leading edge
c
c   input
c     nw       -   odd number of waveforms in the least squares fit
c     sigw     -   sigma of Gaussian weight function used in fit
c     iw       -   128 samples of the waveform
c     iwf      -   (0) - use waveform  (1) - don't use waveform
c     t0       -   initial estimate of arrival time (1-128)
c     sig0     -   initial estimate of pulse width
c     npar     -   (2) - solve for amp and t1 (3) - also solve for sig1
c
c   output
c     rmw      -   model waveform
c     t1       -   updated arrival time (1-128)
c     sig1     -   updated pulse width
c     amp      -   model amplitude
c     std      -   standard deviation of misfit (< 0 -  something went wrong)
c     prob     -   F-test probability that variance reduction is due
c                  to random fluctuations (< 0.2 keep iterating)
c
      implicit real*8(a-h, o-z)
      parameter (mx=11*128)
      integer*4 iw(*),iwf(*)
      real*8    rmw(1)
      real*8    rm(128),dmdt(128),dmds(128),ex(128)
      real*8    t0,t1,sig0,sig1,amp,chi1,chi2
      real*8    a(mx,3),b(mx),w(mx),ata(3,3),atb(3)
      character uplo
c
c  make sure there is enough memory
c
      if(nw*128.gt.mx) then
        write(*,'(a)')'***** increase mx in erf_retrack '
        stop
      endif
c
c   set some constants
c
      damp=0.9d0
      pi=4.*atan(1.d0)
      sq2=sqrt(2.d0)
      sqp=sqrt(pi)
c
c parameters specific to Cryosat LRM data
c
      alph=0.0130d0
      mingate=10
      maxgate=110
c
c  don't let the swh get less than .54 m
c
      if(sig0.le.0.3) sig0=.3
c
c  construct the initial model and partial derivatives
c
      do 10 m=1,128
        time=m
        talph=max(0.,(time-t0+8)*alph)
        eta=(time-t0)/(sq2*sig0)
        ef=erf(eta)
        ex(m)=exp(-talph)
        ex2=exp(-eta*eta)
        rm(m)=ex(m)*(1.+ef)/2.
        dmdt(m)=-ex2/(sig0*sq2*sqp)
        dmds(m)=-eta*ex2/(sig0*sqp)
  10  continue
c
c   estimate the amplitude
c
      sumw=0.
      summ=0.
      nc=(nw+1)/2
      sigw2=sigw**2
      do 13 n=1,nw
c
c  this weight function depends on the distance to the center
c  of the window. also make this large if this waveform is flagged
c  as bad
c
        wfac=exp((n-nc)**2/(2.*sigw2))
        if(iwf(n).eq.1) wfac=100000.
c
        do 12 m=mingate,maxgate
          k=128*(n-1)+m
c
c   The division by 6 is to get a correct chi squared.  there
c   are 100 waveforms average for each 20 Hz waveform.  However
c   only 36(??) of these are independent. Lets guess 72.
c
          w(k)=wfac*(iw(k)+10000)/8.48
          sumw=sumw+iw(k)*rm(m)/(w(k)*w(k))
          summ=summ+rm(m)*rm(m)/(w(k)*w(k))
  12    continue
  13  continue
      amp=sumw/summ
c
c  modify and subtract the starting model and construct the a-matrix
c
      chi1=0.
      do 16 n=1,nw
        do 15 m=mingate,maxgate
          k=128*(n-1)+m
          b(k)=(iw(k)-amp*rm(m))/w(k)
          chi1=chi1+b(k)**2
          a(k,1)=rm(m)/w(k)
          a(k,2)=amp*dmdt(m)/w(k)
          a(k,3)=amp*dmds(m)/w(k)
  15    continue
  16  continue
c
c  compute ata and atb
c
      do 20 i=1,npar
c
c  atb
c
        sumb=0.
        do 17 n=1,nw
        do 17 m=mingate,maxgate
          k=128*(n-1)+m
          sumb=sumb+a(k,i)*b(k)
  17    continue
        atb(i)=sumb
c
c  ata
c
        do 19 j=i,npar
          suma=0.
          do 18 n=1,nw
          do 18 m=mingate,maxgate
            k=128*(n-1)+m
            suma=suma+a(k,i)*a(k,j)
  18      continue
          ata(i,j)=suma
  19    continue
  20  continue
c
c  solve the problem
c
      uplo='U'
      np=3
      nt=1
      call dposv(uplo,npar,nt,ata,np,atb,np,info)
c
      if(info.eq.0) then
c
c  construct the solution
c
        amp=amp+damp*atb(1)
        t1=t0+damp*atb(2)
        if(npar.eq.2) then
          sig1=sig0
        else
          sig1=sig0+damp*atb(3)
        endif
c
c   check for absurd waveforms
c
        if(abs(t1-32.5).gt.20.0.or.amp.lt.5000.0
     +    .or.amp.gt.200000.or.sig1.lt.0.1) then
          std=-9.
          return
        endif
c
c   generate the final model
c
        do 30 m=1,128
          time=m
          talph=max(0.,(time-t1+8)*alph)
          ex(m)=exp(-talph)
          eta=(time-t1)/(sq2*sig1)
          ef=erf(eta)
          rm(m)=ex(m)*(1.+ef)/2.
          rmw(m)=rm(m)*amp
  30    continue
c
c   evaluate the misfit
c
        chi2=0.
        ndata=0
        do 40 n=1,nw
        do 40 m=mingate,maxgate
          k=128*(n-1)+m
          ndata=ndata+1
          chi2=chi2+((iw(k)-rmw(m))/w(k))**2
  40    continue
        std=sqrt(chi2/(ndata-npar))
        f=chi2/chi1
        z=(f-1.046511628)/0.323222961
        prob=(1.+erf(z*sq2))/2.
      else
        std=-abs(info)
      endif
      return
      end
